#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
#################################################################################
# Pyfloppy
# copyright (c) 2003 - 2006 Michael Lange <klappnase@8ung.at>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################################
import Tkinter
Tkinter.wantobjects = 0
from Tkinter import *
import ScrolledText
import tkMessageBox
import fcntl
import os
import sys
import popen2
import fileinput
import gettext

programpath = os.path.abspath(sys.path[0])
if programpath.startswith('/usr/local'):
    localepath = '/usr/local/share/locale'
else:
    localepath = '/usr/share/locale'
gettext.install('pyfloppy', localepath)

##################################################################################################################
#                                                                                                                #
##################################################################################################################

class ProcessHandler:
    def __init__(self, tkroot):
        self.tkroot = tkroot
        self._process = None
        self._runcmd = None
        self._stopcmd = None
    
    def run(self, command, runcmd, stopcmd, *args):
        if self._process:
            return
        self._runcmd = runcmd
        self._stopcmd = stopcmd
        cmd = ' '.join(('exec', command, ' '.join(*args)))
        self._process = proc = popen2.Popen4(cmd)
        fileobject = proc.fromchild
        fcntl.fcntl(fileobject.fileno(), fcntl.F_SETFL, os.O_NONBLOCK)
        self._createfilehandler()
        #print 'Starting process "%s" at PID %d.' % (command, proc.pid)
    
    def _createfilehandler(self):
        # seems like tk.createfilehandler() is broken when run with threaded tcl
        # at least it crashes on my debian 3.1
        # don't bother testing if our tcl actually is threaded -- too dangerous
        # our workaround seems to be good enough to replace createfilehandler() anyway
        self.tkroot.after(200, self._msghandler_loop)
        #self.tkroot.tk.createfilehandler(self._process.fromchild, Tkinter.READABLE, self._msghandler)
    
    def _msghandler(self, fileobject):
        p = self._process.poll()
        r = self._runcmd
        if r:
            try:
                msg = fileobject.read()
            except IOError:
                msg = ''
            if msg:
                r(msg)
        if  p != -1:
            self.stop()
        
    def _msghandler_loop(self):
        try:
            self._msghandler(self._process.fromchild)
            self.tkroot.after(100, self._msghandler_loop)
        except AttributeError:
            # process has finished
            pass
    
    def kill(self, signal=15):
        if self._process:
            pid = self._process.pid
            try:
                os.kill(pid, signal)
                print 'Process %d killed with signal %d' %(pid, signal)
            except OSError:
                print 'Warning: unable to kill process %d.' % pid
    
    def stop(self):
        if self._process:
            p = self._process.poll()
            s = self._stopcmd
            #self.tkroot.tk.deletefilehandler(self._process.fromchild)
            self._process = None
            self._runcmd = None
            self._stopcmd = None
            if s:
                s(p)
    
    def find(self, *alternatives):
        paths = os.getenv('PATH').split(':')
        for p in ['/sbin', '/bin', '/usr/sbin', '/usr/bin', '/usr/local/sbin', '/usr/local/sbin']:
            if not p in paths:
                paths.append(p)
        for a in alternatives:
            for p in paths:
                full = os.path.join(p, a)
                if os.path.exists(full) and os.access(full, os.X_OK):
                    return full
            print 'Warning: no "%s" in %s' % (a, ':'.join(paths))
            tkMessageBox.showerror(master=self.tkroot, title='pyfloppy',
                                   message=_('Unable to find "%s", please check your installation.') % a)
        return ''
    
    def busy(self):
        return self._process

##################################################################################################################
#                                                                                                                #
##################################################################################################################
class OutputWindow(ScrolledText.ScrolledText):    
    def __init__(self, master, *args, **kw):
        ScrolledText.ScrolledText.__init__(self, master, state='disabled', wrap='word', takefocus=0, cursor='', *args, **kw)
        self.frame.configure(relief='groove', bd=2, highlightthickness=1, highlightbackground=self.frame['bg'])
        self.vbar.configure(highlightthickness=0, takefocus=1, width=12)
        self.vbar.bind('<FocusIn>', lambda event: self.frame.configure(highlightbackground=self.frame['highlightcolor']))
        self.vbar.bind('<FocusOut>', lambda event: self.frame.configure(highlightbackground=self.frame['bg']))
        self._id1 = self.bind('<1>')
        self._id2 = self.bind('<B1-Motion>')
        self._id3 = self.bind('<Leave>')
        self.bind('<Configure>', self._configure_lines)
        self.tag_config('bold', font=('Courier', '-12', 'bold'))
    
    def append(self, what, tag='normal'):
        '''Inserts text WHAT into the text widget and takes care to properly handle
        backspace and carriage return characters to keep the formatting of a console
        command's output messages intact.'''
        lines = what.split('\n')
        lastline = len(lines) - 1
        newlines = []
        cur_line = 0
        leading_bs = 0
        trailing_bs = 0
        leading_cr = 0
        trailing_cr = 0
        for line in lines:
            new = ''
            insert = 0
            for char in line:
                if not char in ('\b', '\r'):
                    new = ''.join((new[:insert], char))
                    insert += 1
                    trailing_bs = 0
                    trailing_cr = 0
                elif char == '\b':
                    if insert > 0:
                        insert -= 1
                    elif cur_line == 0:
                        leading_bs += 1
                    if cur_line == lastline:
                        trailing_bs += 1
                elif char == '\r':
                    if insert > 0:
                        insert = 0
                    elif cur_line == 0:
                        leading_cr = 1
                    if cur_line == lastline:
                        trailing_cr = 1
            newlines.append(new)
            cur_line += 1
        newtext = '\n'.join(newlines)
        # now update the Text; if newtext starts with a newline, keep the already
        # inserted text intact, else adjust the insertion mark with respect to
        # leading backspace or carriage return chars and then delete everything
        # after the insertion cursor first to override the old text
        self.config(state='normal')
        if newtext.startswith('\n'):
            oldinsert = self.index('insert').split('.')
            self.mark_set('insert', '.'.join((oldinsert[0], 'end')))
        else:
            newinsert = self._adjust_insert_mark(leading_bs, leading_cr)
            self.delete('insert', '.'.join((newinsert[0], 'end')))
        self.insert('insert', newtext, tag)
        self.config(state='disabled')
        self._adjust_insert_mark(trailing_bs, trailing_cr)
        self.see('end')
    
    def append_command(self, command, *args):
        msg = '\n$$$ %s ' % command
        if args:
            msg += ' '.join(*args)
        msg += '\n\n'
        self.append(msg, 'bold')
    
    def draw_line(self, *args):
        self.append('\n')
        f = Tkinter.Frame(self, height=4, width=self.winfo_width()-20, relief='ridge', bd=2, bg='gray60')
        self.window_create('insert', window=f)
    
    def set_state(self, state='busy'):
        if state == 'busy':
            # prevent mouse clicks from being propagated to the text widget first
            # so the user won't mess up the cursor position while we're busy
            self.bind('<1>', self._ignore_event)
            self.bind('<B1-Motion>', self._ignore_event)
            self.bind('<Leave>', self._ignore_event)
            self.mark_set('insert', 'end')
        elif state == 'idle':
            # restore the original bindings to enable copy'n'paste from the text
            self.bind('<1>', self._id1)
            self.bind('<B1-Motion>', self._id2)
            self.bind('<Leave>', self._id3)
    
    def _adjust_insert_mark(self, offset, cr):
        '''Move the insertion cursor by OFFSET characters to the left; if CR is 1,
        move the insertion cursor to the beginning of the current line.'''
        oldinsert = self.index('insert').split('.')
        if offset or cr:
            oldinsert_int = map(int, oldinsert)
            if oldinsert_int[1] > 0:
                if cr:
                    new = 0
                else:
                    new = oldinsert_int[1] - offset
                    if new < 0:
                        new = 0
                oldinsert = map(str, (oldinsert_int[0], new))
                self.mark_set('insert', '.'.join(map(str, oldinsert)))
        return oldinsert

    def _configure_lines(self, *event):
        try:
            for win in self.window_names():
                newwidth = self.winfo_width() - 20
                self._nametowidget(win).configure(width=newwidth)
        except:
            pass
    
    def _ignore_event(self, event):
        return 'break'

##################################################################################################################
#                                                                                                                #
##################################################################################################################

class ConfigHandler:
    def __init__(self):
        self._conffile = os.path.join(os.environ['HOME'], '.pyfloppy')
        self._opts = {'mode':'',
                     'fs':'mke2fs',
                     'dev':'/dev/fd0',
                     'low_lev':0,
                     'disk_type':'3.5" 1440 kB',
                     'verify':1,
                     'e2opts':'-m 0',
                     'dosopts':''}
        self._bool_keys = ('low_lev', 'verify')
        self._read_conffile()

    def _get_key_and_value(self, line):
        try:
            l = line.split('=')
            key = l[0].strip()
            value = l[1].strip()
            return (key, value)
        except:
            return ('', '')
    
    def _read_conffile(self):
        try:
            for line in fileinput.input(self._conffile):
                line = line.strip()
                key, value = self._get_key_and_value(line)
                if self._opts.has_key(key):
                    if key in self._bool_keys:
                        if value in ('1', 'True'):
                            value = 1
                        else:
                            value = 0
                    self._opts[key] = value
        except (IOError, OSError):
            pass
    
    def get(self, opt):
        return self._opts[opt]
    
    def set(self, opt, value):
        self._opts[opt] = value
        
    def save(self):
        f = open(self._conffile, 'w')
        for key in self._opts:
            value = str(self._opts[key])
            f.write('%s=%s\n' %(key, value))
        f.close()

##################################################################################################################
#                                                                                                                #
##################################################################################################################

class Pyfloppy(Frame):
    def __init__(self, master, **kw):
        Frame.__init__(self, master, **kw)
        self.grid_columnconfigure(0, weight=1)
        self.grid_columnconfigure(1, weight=1)
        self.grid_rowconfigure(9, weight=1)

        self.master = master
        self.processhandler = ProcessHandler(master)
        self.confighandler = confighandler = ConfigHandler()
        self.pending = 0
        
        self.mode = StringVar()
        self.mode.set(confighandler.get('mode'))
        self.fs = StringVar()
        self.fs.set(confighandler.get('fs'))
        self.dev = StringVar()
        self.dev.set(confighandler.get('dev'))
        self.lowlevel = IntVar()
        self.lowlevel.set(confighandler.get('low_lev'))
        self.disk_type = StringVar()
        self.disk_type.set(confighandler.get('disk_type'))
        self.verify = IntVar()
        self.verify.set(confighandler.get('verify'))
        self.e2opts = StringVar()
        self.e2opts.set(confighandler.get('e2opts'))
        self.dosopts = StringVar()
        self.dosopts.set(confighandler.get('dosopts'))
        
        ############################################################################################
        
        self.info = PhotoImage(data='R0lGODlhGAAYAOeQAAAAADs4LVdfZldgZlthaFtiaWhjUF5lal9mbGRqb2VqcGt0fJGCTaGRVqSbfpSdpJafprKgX5Si'
                               'rZWjrpqiqJ6lqpuosp2ps6OorZ6qtKCrtaWqrrSqiaCvu6Ovu8SwaaSxvaWyvqayvqu1vqy2vq25xLO5vq+6xbTAybbByrjBy'
                               'tjCdLbE0bfG0rnH1L7Hz8DH0LvK1bzK173L18LK0L3M2MPL0r7N177N2cDN2cLP28LP3MTQ28TQ3MTR3MTR3crQ1cXS3MbT3s'
                               'fT3cnU3sjV38nV3snV4NDU2crW4MrW4czW4MvX4cvX4s3Y4c3Y483Z487Z48/Z4c/Z4s/Z48/a49Da4tDa5M/b48/b5NDb5dH'
                               'b5dHb5tLc5dPc5tLd59Pd59Te6NXe59Xf59bf59bf6NXg6dbg5+rft9jh6dnh6Nnh6drh6Nri6tvi6drj6tvj6tvk6tzk693k'
                               '6t3l6t7l697l7N/l69/l7ODm7N/n7eDn7PbpvePq7+Tq8Pjuzenv8vXw3+3x9O3x9e7y9e/y9u/z9fL1+PP1+PP2+Pb4+vz47'
                               'Pj5+/n7+/r7/P39+/////////////////////////////////////////////////////////////////////////////////'
                               '/////////////////////////////////////////////////////////////////////////////////////////////////'
                               '/////////////////////////////////////////////////////////////////////////////////////////////////'
                               '/////////////////////////////////////////////////////////////////////////////////////////////////'
                               '////////////////////////////////////////////////////////////////////////////yH+iVRvb2xiYXItc2l6ZW'
                               'QgaWNvbgo9PT09PT09PT09PT0KCihjKSAyMDA0IEpha3ViICdqaW1tYWMnIFN0ZWluZXIsIApodHRwOi8vamltbWFjLm11c2l'
                               'jaGFsbC5jegoKY3JlYXRlZCB3aXRoIHRoZSBHSU1QLApodHRwOi8vd3d3LmdpbXAub3JnACH5BAEKAP8ALAAAAAAYABgAAAj+'
                               'AP8JHEiwoMGDBQEoVIiwIQAFKYoc8XECAYCGBQGQKOMmzJMkQnRoAIBRIIARbwYhMgTnCZMiOS4AKJkADCBFjRgJEhMFC5EaB'
                               'QA0BIBiTSFGjhgRSpNlTJceIgA0BFBkS59Eig754XKljZolLgA0BHAEyhc9SDZQgPACDx0rMgA0BPCjiRczMLREqVJhzxwpLQ'
                               'A0BFBCCRk5cciIsYEhDxsjHgBgPOBjSps6dvIAAeLGyowBADACyMCDyhk4d0zQcIJDAoCS/wBYuLFjiIoHQWJMAABbIAACIFi'
                               'EWNBBAIDeBBcuRD4QAIcPDRg4AMBcIIA/j/jwWQGgukADHxobNDAAwDuAQIvQoIkAwPu/AMoBuP+3cL59+wEBADs=')
        self.logo = PhotoImage(data='R0lGODlhGAAYAOfaAAAAAAwNDygyPjM4PDY8QTc8QDE+STg+Qzg/QzlCSzRGTjZIUz5HTUBHTTZKWUFKTkJKUU1JQkNM'
                               'VE5KQzpRXU9MREVPVTtTYFFORkdRWFJPR0pTWUtVXUVcbU9bYkZecFJbY1FcZE5eakpgcEhhc0hidkphdlRgaVZhaEtleldja'
                               'mBfaE1meFhja05oeWJhaXNbW2NjY01pf09pfE5pf1pmb2dlXk1sgFxpcVJsgGhmcE9vhGlpaV9sdltufFRwhlZxh2JveVRzjG'
                               'tvcm9vb3Nyd3Nzc293fnV1enp6emWFnMNkWcNlWMJlW8RnW7psam6JnsFqYr9rZGmLo3CKoWyMpHeJmWuNpmyOpm+Pp3KPpG+'
                               'QrHqOmnCRqnGSrs5yZc9yZHKUrXSUr851Z3iXs8x6cMt6c3mYsst7c8t8dMx8csx8c8t9c5eTjNp4aHubstx6aNR9bsiBe36c'
                               'tZuWjn+dtn2gtoGettSBc4Kft4OguIKgu3+iuYOhttyBceCAboWhuOCCboaiuYSjv4ejt4ejuoWkv4iku4qlu6SgmYinw4ynv'
                               'YerwYqqxeCNfrizq6S5y7i1sKG+1pvD2KHB2qPD28C8taTE3r+/v8K/uavE4qrG3ajI4anJ4qrJ4arJ4sfEvqnN5em3rcfFv8'
                               'XFxbPK3KrO4+m4r67M5cnGwOm5scrGwLXK5bPN48vIwrrM3LDR5MzJw+W/uu68ss3Kxu6+te2/trfS57bT583NzbnT5rnT5+7'
                               'BuM/Oyc7OztHOybrV6bvW6O/Fu/DGvPHIvtbU0MPX9NfU0NfV0dnX1NrY09za193b19/d2uDe2uHg3OLh3ePh3uTi4Obl4ufm'
                               '4uno5urp5+vq6O3s6vDv7P///////////////////////////////////////////////////////////////////////////'
                               '////////////////////////////////////////////////////////////////////////////yH+FUNyZWF0ZWQgd2l0aC'
                               'BUaGUgR0lNUAAh+QQBCgD/ACwAAAAAGAAYAAAI/gD/CRxIsKDBggASKlzIcCFBAMZCIRFGbNisWrxsiTol6pQqWUVaSQIwEEC'
                               'wRiscBfoDx48bPHG+gBnjhMkLQkAADAQATJEOOWnMoFHDRkqZKGuaLHmi484PAAMB4GJkRJvVq1ivGtGTA8BAALsMGclK1qoR'
                               'PTMADASga5ARX7niyp2bC5ORQyUADARwa0+MsmSNHPoAYCAAVnyMAM5qpFAHAAMBoLJjxBfdy6SMFDoBYCAAWH1gLMZqRE8IA'
                               'AMBmHrDJYmR16+HEIFthMcRQRQADATQScwdPYLy1JmT54wgQIAK5VmkB5EDAAMBaAqzxcsDCCCC9MBRo4UKEBcStpCBtADAQA'
                               'CfumTZ0KZZtGvXrDXrlYoOihRCNikAMBCAJyxVNKAMNTZooEEyo1jySC8SyLBDKQYAMBAAnFyhRQPQZGODgbRkYskqy3BAww6'
                               'vGADAQABcMgUVBzhzDQYVVACKJa48U00GLtxQCQIADASAFVAoMcAxxUwQQQSW/DINNtNYQAILlBQAAEEBiOCDB4lY8kgssSAj'
                               'jTXMRMKACSNMQgAABS2k5poACADAQXDGKeecBgUEADs=')
        titleframe = Frame(self)
        titleframe.grid(row=0, column=0, columnspan=2, pady=5, sticky='ew')
        titleframe.grid_columnconfigure(1, weight=1)
        infobutton = Button(titleframe, width=24, height=24, relief='flat', image=self.info, command=self.show_info, overrelief='raised')
        infobutton.grid(row=0, column=0, padx=10)
        Label(titleframe, text=_('Format floppy disk'), font=('helvetica', '-18', 'bold')).grid(row=0, column=1)
        Label(titleframe, image=self.logo).grid(row=0, column=2, padx=10)
        
        ###################################################################################
        
        Label(self, text=_('Device :')).grid(row=1, column=0, sticky='w', padx=10, pady=5)
        devicemenu = OptionMenu(self, self.dev, '/dev/fd0', '/dev/fd1', '/dev/floppy/0', '/dev/floppy/1')
        devicemenu.grid(row=1, column=0, columnspan=2, sticky='e', padx=10, pady=5)
        devicemenu.configure(width=12, takefocus=1)
        
        ###################################################################################
        
        lowlevelframe = Frame(self, relief='groove', bd=2)
        lowlevelframe.grid(row=2, column=0, columnspan=2, sticky='ew', padx=5, pady=5, ipadx=5, ipady=5)
        lowlevelframe.grid_columnconfigure(0, weight=1)
        lowlevelbutton = Checkbutton(lowlevelframe, text=_('Low-level-formatting'), variable=self.lowlevel, command=self.toggle_lowlevel,
                                     onvalue=1, offvalue=0)
        lowlevelbutton.grid(row=0, column=0, columnspan=2, sticky='w', padx=10)
        self.typelabel = Label(lowlevelframe, text=_('Disk type :'))
        self.typelabel.grid(row=1, column=0, sticky='w', padx=20)
        self.typemenu = OptionMenu(lowlevelframe, self.disk_type, '3.5" 1440 kB', '3.5" 720 kB',
                                  '3.5" 360 kB', '5.25" 1200 kB', '5.25" 720 kB', '5.25" 360 kB')
        self.typemenu.grid(row=1,  column=0, columnspan=2, sticky='e', padx=20)
        self.typemenu.configure(width=12, takefocus=1)
        self.verifybutton = Checkbutton(lowlevelframe, text=_('Verify file system'), variable=self.verify, onvalue=1, offvalue=0)
        self.verifybutton.grid(row=2, column=0, columnspan=2, sticky='w', padx=20)
        
        ###################################################################################
        
        fsframe = Frame(self)
        fsframe.grid(row=3, column=0, columnspan=2, sticky='ew', padx=10, pady=5)
        fsframe.grid_columnconfigure(1, weight=1)
        Label(fsframe, text=_('File system :')).grid(row=0, column=0, sticky='w')
        e2fsbutton = Radiobutton(fsframe, variable=self.fs, value='mke2fs', text=_('ext2'), command=self.toggle_fs)
        e2fsbutton.grid(row=0, column=1)
        dosfsbutton = Radiobutton(fsframe, variable=self.fs, value='mkdosfs', text=_('dos'), command=self.toggle_fs)
        dosfsbutton.grid(row=0, column=2, padx=20)
        
        ###################################################################################
        
        Label(self, text=_('Bad blocks check :')).grid(row=4, column=0, columnspan=2, sticky='w', padx=10, pady=5)
        badblocksframe = Frame(self)
        badblocksframe.grid(row=5, column=0, columnspan=2, sticky='ew', padx=10)
        badblocksframe.grid_columnconfigure(1, weight=1)
        nonebutton = Radiobutton(badblocksframe, variable=self.mode, value='', text=_('None'))
        nonebutton.grid(row=0, column=0, padx=10)
        readbutton = Radiobutton(badblocksframe, variable=self.mode, value='-c', text=_('Read') +  u' ( -c )')
        readbutton.grid(row=0, column=1)
        self.readwritebutton = Radiobutton(badblocksframe, variable=self.mode, value='-cc', text=_('Read/write') + u' ( -cc )')
        self.readwritebutton.grid(row=0, column=2, padx=10)
        
        ###################################################################################
        
        optsframe = Frame(self)
        optsframe.grid(row=6, column=0, columnspan=2, sticky='ew')
        optsframe.grid_columnconfigure(1, weight=1)
        Label(optsframe, text=_('Further options :')).grid(row=0, column=0, padx=10, pady=5)
        self.optsentry = Entry(optsframe, bg='white')
        self.optsentry.grid(row=0, column=1, sticky='ew', padx=10, pady=5)
        
        self.format_icon = PhotoImage(data='R0lGODlhEAAQAKU6AAAAAAACAAAEAAAQAAASAAEbAQEhAgEjAgEnAgIuAwI0BAI6BgRICgZMCQRSDQV'
                                      'VDwdXDQVdEQZgEwhgEAZiEwdtGAhwGQl5Hg99HAyOLg6SMA+bNxOrRROvShW1USC0Rhe5VBe7WRnHYxvJaRz'
                                      'KaijciS/bjDHbjT/XijjakEDYj0LYk0fXklPdmlbgnVjgolvhoGvprHDqr3bsrofxvojyv5L0xZj2yav508j'
                                      '84////////////////////////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQBCgA/ACwAAAAAEAAQAAA'
                                      'Ga8CfcEj8AQDFpDCAEQGURMBHZgBAhxOcBkD0JIqNmeoAIDJykSEAVdsAkqZb5QeA0E4LQLIAc1EKMS0cAlA'
                                      'SLysZNyUPAFACIyw2KyAEVz8KKSQhFgCWPxUdFwgAngMOCACeQgFHqkIArp5BADs=')
        self.cancelicon = PhotoImage(data='R0lGODlhEAAQAMZlAAEBAQICAgUFBQgFBQkFBBIFAgkJCQ8JBw4LCRsHBA0NDRgJBhwJBSAIBCAL'
                                     'CCMKBh0PDCQNCSwLBSwOCSYRDSYRDi8OByUSDysTDiwUDy8VEC0XFDEXEjIYEzAZFDsXETQaFTgZFDwYEk'
                                     'EXEDkcF0kWDE4WDT4eGEwbE0IgGU4dFFchGVwlG4ImF5IsG5EvHpIvHpYuHJQwHpkvHZYxHpgxHqE1IaI6'
                                     'Jq02ILA3ILI4IbQ5IrY5IrQ6I7c5Irk6Iro6Irk7JLo7Irs7IrY9Jrw7Ir07Ir47Ir87Ir08JLg+J8A7Is'
                                     'I7Ir09Jb0/KL4/J61GMsJBKcNMNsRNNsZNN8hON7RbSs5SOslWQdBfSdNjTs5mU8FuXsptW8xyYc1yYdZv'
                                     'W9lwW8t4aNF/cN2Nff/////////////////////////////////////////////////////////////////'
                                     '//////////////////////////////////////////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQBCg'
                                     'B/ACwAAAAAEAAQAAAHo4B/goOEhYMpJAqGfwECgx1kWSqGG1xQBIInWlhUKIQeYVExBYIIUktIUyyCEGBBL'
                                     'iaEGlJHRzofAl1KLRKGGFJGQj5WVzMNi38iUkBAVTwWAcgBJVtJQEM7GcgJY09EPkVGORiGIWJOMAYfUrU9F'
                                     '4MAK14/NA6CI1NISzcDgg9fTWpEIISCChMcFAQdeCFjQSEFKqrYqDBoAoNFCkBwQMbxTyAAOw==')
        self.exiticon = PhotoImage(data='R0lGODlhEAAQAKU+AAAAAAYIBgcJBw4RDA8SDRYcFBkfFh0lGyAoHYgAACcxJJkAACo1Jis3Jy06KTE9L'
                                    'KEQCzRBLzVDMaMUDrAWCjtKNbEbEEJUPVFUT0lcQpk5KUldQ09kSFtbWqo/LFxgWVNoS1RqTcg+K7FNNmxsat9'
                                    'CHrNTPnl5d+RiRY2Ni5KSkZOUked1W5qamJycm6Skoq2tq7S0sre3tbm5t729u8DAvcHBvsnJx9DQzdPT0NTU0'
                                    'd7e3ODg3uLi4P///////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQBCgA/ACwAAAAAEAAQAAAGhcCfcEg'
                                    'sGgHIpPLYy9VgquUw2azJWlIhAAXY5WwzFyAgAPyULBMABxYPCOYtal4aAWgvccEQR1H+FBYaACcdAAcIfQuLC'
                                    'wkeACRiCgxxIhATCY4AbQAND3FJmjqcDBJmU489ozEpAA4Vp1pIPDcrGAARFxuxUzIfABUZHCAhvLJKWUbKP0EAOw==')
        bframe = Frame(self)
        bframe.grid(row=7, column=0, columnspan=2, padx=8, sticky='ew')
        formatbutton = Button(bframe, text=_('Format'), command=self.start, image=self.format_icon, compound='left')
        formatbutton.grid(row=7, column=0, padx=5, pady=8, sticky='ew')
        self.closebutton = Button(bframe, text=_('Close'), command=self.close, image=self.exiticon, compound='left')
        self.closebutton.grid(row=7, column=1, padx=5, pady=8, sticky='ew')
        # make both buttons have the same width
        self.update_idletasks()
        max = 0
        for b in (formatbutton, self.closebutton):
            if b.winfo_reqwidth() + 16 > max:
                max = b.winfo_reqwidth() + 16
        for c in range(2):
            bframe.grid_columnconfigure(c, weight=1, minsize=max)
        master.minsize(width=self.winfo_reqwidth(), height=450)
        
        ###################################################################################
        Label(self, text=_('Command line output:')).grid(row=8, column=0,
                                                        columnspan=2, padx=5, pady=(5, 0), sticky='w')
        self.outputwindow = OutputWindow(self, width=5, height=10)
        self.outputwindow.grid(row=9, column=0, columnspan=2, padx=3, pady=3, sticky='news')
        
        ###################################################################################
        
        self.widgets = (devicemenu, lowlevelbutton, self.typemenu, self.verifybutton, e2fsbutton,
                        dosfsbutton, nonebutton, readbutton, self.readwritebutton, self.optsentry,
                        formatbutton)
        
        self.toggle_fs()
        self.toggle_lowlevel()
    
    def start(self):
        if not self.processhandler.busy():
            self.closebutton.configure(text=_('Cancel'), command=self.cancel, image=self.cancelicon)
            self.outputwindow.set_state('busy')
            for w in self.widgets:
                w.config(state='disabled')
            if self.lowlevel.get():
                self.pending = 1
                self.format_low_level()
            else:
                self.format()
    
    def format(self):
        mkfs_cmd = self.processhandler.find(self.fs.get())
        if not mkfs_cmd:
            return self.reset('command not found')
        if self.fs.get() == 'mke2fs':
            args = self.e2opts.get().split()
        else:
            args = ['-v'] + self.dosopts.get().split()
        args = args + [self.mode.get(), self.dev.get()]
        self.outputwindow.append_command(mkfs_cmd, args)
        self.processhandler.run(mkfs_cmd, self.outputwindow.append, self.reset, args)
    
    def format_low_level(self):
        fdformat = self.processhandler.find('fdformat')
        if not fdformat:
            return self.reset('command not found')
        args = []
        if not self.verify.get():
            args.append('-n')
        args.append(self.get_device_node())
        self.outputwindow.append_command(fdformat, args)
        self.processhandler.run(fdformat, self.outputwindow.append, self.reset, args)
        
    def get_device_node(self):
        d = self.disk_type.get().split()
        s = d[1]
        if s == '1440' or s == '1200':
            t = 'H'
        elif s == '720':
            t = 'Q'
        elif s == '360':
            t = 'D'
        if d[0] == '5.25"':
            t = t.lower()
        dev_node = self.dev.get() + t + s
        if not os.path.exists(dev_node):
            dev_node = self.dev.get() + 'u' + s
        if not os.path.exists(dev_node):
            dev_node = self.dev.get()
        return dev_node
        
    def toggle_fs(self):
        if self.fs.get() == 'mke2fs':
            self.optsentry.configure(textvariable=self.e2opts)
            self.readwritebutton.configure(state='normal')
        else:
            self.optsentry.configure(textvariable=self.dosopts)
            if self.mode.get() == '-cc':
                self.mode.set('-c')
            self.readwritebutton.configure(state='disabled')
    
    def toggle_lowlevel(self):
        if self.lowlevel.get():
            self.typemenu.configure(state='normal')
            self.typelabel.configure(state='normal')
            self.verifybutton.configure(state='normal')
        else:
            self.typemenu.configure(state='disabled')
            self.typelabel.configure(state='disabled')
            self.verifybutton.configure(state='disabled')

    def reset(self, exit_state):
        if exit_state != 'command not found':
            self.outputwindow.append('\n$$$ Exit %d\n' % exit_state, 'bold')
        if exit_state != 0:
            # don't start mke2fs or mkdosfs if fdformat failed
            self.pending = 0
        if self.pending:
            self.pending = 0
            self.format()
            return
        self.outputwindow.draw_line()
        self.outputwindow.append('\n')
        self.outputwindow.set_state('idle')
        self.closebutton.configure(text=_('Close'), command=self.close, image=self.exiticon)
        for w in self.widgets:
            w.config(state='normal')
        self.toggle_fs()
        self.toggle_lowlevel()
        if exit_state == 0:
            tkMessageBox.showinfo(master=self.master, title='pyfloppy', message=_('Formatting successfully finished.'))
        elif exit_state == 9:
            tkMessageBox.showwarning(master=self.master, title='pyfloppy', message=_('Formatting process aborted.'))
        elif exit_state != 'command not found':
            tkMessageBox.showerror(master=self.master, title='pyfloppy',
                                   message=_('An error occured while formatting (exit %d).') % exit_state)

    def show_info(self):
        tkMessageBox.showinfo(title='pyfloppy', message=_('PyFloppy - by Michael Lange\nContact : <klappnase@8ung.at>'))
    
    def cancel(self):
        self.processhandler.kill(9)
    
    def close(self):
        if self.processhandler.busy():
            return self.bell()
        self.confighandler.set('mode', self.mode.get())
        self.confighandler.set('fs', self.fs.get())
        self.confighandler.set('dev', self.dev.get())
        self.confighandler.set('low_lev', self.lowlevel.get())
        self.confighandler.set('disk_type', self.disk_type.get())
        self.confighandler.set('verify', self.verify.get())
        self.confighandler.set('e2opts', self.e2opts.get())
        self.confighandler.set('dosopts', self.dosopts.get())
        self.confighandler.save()
        self.master.quit()

##################################################################################################################
#                                                                                                                #
##################################################################################################################

optionDB = {'*font': 'helvetica -12',
            '*selectBackground': 'blue4',
            '*selectForeground': 'white',
            '*Text*font' : 'Courier -12',
            '*Text*background' : 'white'}

def main():
    root = Tk(className='pyfloppy')
    root.withdraw()
    root.bind_class('Button', '<Return>', lambda event: root.event_generate('<space>'))
    root.bind_class('Menubutton', '<Return>', lambda event: root.event_generate('<space>'))
    for opt in optionDB:
        root.option_add(opt, optionDB[opt])
    p = Pyfloppy(root, relief='ridge', bd=3)
    p.pack(fill='both', expand=1)
    root.protocol('WM_DELETE_WINDOW', p.close)
    root.update_idletasks()
    root.deiconify()
    root.mainloop()
    root.destroy()

if __name__ == '__main__':
    main()
